/*
  ==============================================================================

   This file is part of the JUCE tutorials.
   Copyright (c) 2020 - Raw Material Software Limited

   The code included in this file is provided under the terms of the ISC license
   http://www.isc.org/downloads/software-support-policy/isc-license. Permission
   To use, copy, modify, and/or distribute this software for any purpose with or
   without fee is hereby granted provided that the above copyright notice and
   this permission notice appear in all copies.

   THE SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY, AND ALL WARRANTIES,
   WHETHER EXPRESSED OR IMPLIED, INCLUDING MERCHANTABILITY AND FITNESS FOR
   PURPOSE, ARE DISCLAIMED.

  ==============================================================================
*/

/*******************************************************************************
 The block below describes the properties of this PIP. A PIP is a short snippet
 of code that can be read by the Projucer and used to generate a JUCE project.

 BEGIN_JUCE_PIP_METADATA

 name:             OnlineUnlockStatusTutorial
 version:          1.0.0
 vendor:           JUCE
 website:          http://juce.com
 description:      Showcases online plugin registration.

 dependencies:     juce_core, juce_cryptography, juce_data_structures,
                   juce_events, juce_graphics, juce_gui_basics, juce_gui_extra,
                   juce_product_unlocking
 exporters:        xcode_mac, vs2019, linux_make

 type:             Component
 mainClass:        MainContentComponent

 useLocalCopy:     1

 END_JUCE_PIP_METADATA

*******************************************************************************/


#pragma once

//==============================================================================
class TutorialMarketplaceStatus  : public juce::OnlineUnlockStatus
{
public:
    TutorialMarketplaceStatus() = default;

    juce::String getProductID() override { return {}; }
    bool doesProductIDMatch (const juce::String&) override { return false; }

    juce::RSAKey getPublicKey() override { return {}; }

    void saveState (const juce::String&) override {}
    juce::String getState() override { return {}; }

    juce::String getWebsiteName() override { return {}; }
    juce::URL getServerAuthenticationURL() override { return {}; }

    juce::String readReplyFromWebserver (const juce::String& email, const juce::String& password) override
    {
        juce::URL url (getServerAuthenticationURL()
                    .withParameter ("product", getProductID())
                    .withParameter ("email", email)
                    .withParameter ("pw", password)
                    .withParameter ("os", juce::SystemStats::getOperatingSystemName())
                    .withParameter ("mach", getLocalMachineIDs()[0]));

        DBG ("Trying to unlock via URL: " << url.toString (true));

        {
            juce::ScopedLock lock (streamCreationLock);
            stream.reset (new juce::WebInputStream (url, true));
        }

        if (stream->connect (nullptr))
        {
            auto* thread = juce::Thread::getCurrentThread();

            if (thread->threadShouldExit() || stream->isError())
                return {};

            auto contentLength = stream->getTotalLength();
            auto downloaded    = 0;

            const size_t bufferSize = 0x8000;
            juce::HeapBlock<char> buffer (bufferSize);

            while (! (stream->isExhausted() || stream->isError() || thread->threadShouldExit()))
            {
                auto max = juce::jmin ((int) bufferSize, contentLength < 0 ? std::numeric_limits<int>::max()
                                                                           : static_cast<int> (contentLength - downloaded));

                auto actualBytesRead = stream->read (buffer.get() + downloaded, max - downloaded);

                if (actualBytesRead < 0 || thread->threadShouldExit() || stream->isError())
                    break;

                downloaded += actualBytesRead;

                if (downloaded == contentLength)
                    break;
            }

            if (thread->threadShouldExit() || stream->isError() || (contentLength > 0 && downloaded < contentLength))
                return {};

            return { juce::CharPointer_UTF8 (buffer.get()) };
        }

        return {};
    }

    void userCancelled() override
    {
        juce::ScopedLock lock (streamCreationLock);

        if (stream != nullptr)
            stream->cancel();
    }

private:
    juce::CriticalSection streamCreationLock;
    std::unique_ptr<juce::WebInputStream> stream;
};

//==============================================================================
class MainContentComponent   : public juce::Component
{
public:
    //==============================================================================
    MainContentComponent()
    {
        addAndMakeVisible (unlockLabel);
        unlockLabel.setFont (juce::Font (15.0f, juce::Font::bold));
        unlockLabel.setColour (juce::Label::textColourId, juce::Colours::red);

        addAndMakeVisible (unlockButton);
        unlockButton.onClick = [this] { showForm(); };

        addAndMakeVisible (secretButton);
        secretButton.setEnabled (false);
        secretButton.onClick = [this] { checkFeature(); };

        setSize (400, 250);
    }

    void paint (juce::Graphics& g) override
    {
        g.fillAll (getLookAndFeel().findColour (juce::ResizableWindow::backgroundColourId));
    }

    void resized() override
    {
        unlockLabel .centreWithSize (100, 40);
        unlockButton.centreWithSize (100, 40);
        secretButton.centreWithSize (100, 40);

        unlockLabel .setBounds (unlockLabel .getBounds().translated (0, -60));
        unlockButton.setBounds (unlockButton.getBounds().translated (0, -10));
        secretButton.setBounds (secretButton.getBounds().translated (0,  40));
    }

private:
    //==============================================================================
    void showForm()
    {

    }

    void checkFeature()
    {

    }

    juce::Label      unlockLabel  { {}, "Status: Locked" };
    juce::TextButton unlockButton { "Unlock" },
                     secretButton { "Super Secret Feature" };

    JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (MainContentComponent)
};
